#define FOF_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/


/*****************************************************************************/

/******************************************************************************

Version Dependant DEFINES
=========================

******************************************************************************/

#define DATE_RELEASE_DAY    09
#define DATE_RELEASE_MONTH  11
#define DATE_RELEASE_YEAR   97

#define FOF_RELEASE_NUMBER 68

#define FOF_VERSION_MAJOR 0
#define FOF_VERSION_MINOR "99a"

#define FOF_VERSION_TYPE  BETA

#include <Xm/Xm.h>
#include <unistd.h>
#include <time.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>

#include "interface.h"
#include "dirlist.h"
#include "dirtree.h"
#include "error_handler.h"
#include "dialogs.h"
#include "menu.h"
#include "config.h"
#include "fileview.h"
#include "usermenu.h"
#include "sort.h"
#include "command.h"
#include "contextmenu.h"
#include "dirhistory.h"
#include "windows.h"
#include "pixmap.h"

#ifdef OFFIX_DND
  #include "offix_dnd.h"
#endif

uid_t UserID;
gid_t GroupID;

int AskRefreshFileList;
int BlockRefreshRoutine;

XtIntervalId TimeOutDirID;
XtIntervalId TimeOutTreeID;
XtIntervalId TimeOutStartID;
unsigned long TimeOutStart;
unsigned long TimeOutDir;
unsigned long TimeOutTreeID;

unsigned long RefreshDirLastTime;

void SetHostValue(void)
{
  Widget w_temp;
  char temp_string[256];
  XmString string;
  int result;
  Arg args[2];
  int nbargs;

  result = gethostname(temp_string, 255);
  if(result==0) {
    w_temp = XtNameToWidget(W_MainWindow, "*HostLabel");
    if(w_temp==NULL) {
      NonFatalError("fof.c", "SetHostValue","Unable to Retrieve HostLabel widget");
      return;
    }
    string = XmStringCreateSimple(temp_string);
    nbargs=0;
    XtSetArg(args[nbargs], XmNlabelString, string);nbargs++;
    XtSetValues(w_temp, args, nbargs);
    XmStringFree(string);
  } else NonFatalError("fof.c", "SetHostValue", "Unable to Get HostName");
}

void SetDateValue(void)
{
  Widget w_temp;
  char temp_string[256];
  time_t t;
  XmString string;
  int result;
  Arg args[2];
  int nbargs;


  t= time(NULL);
  result=strftime(temp_string, sizeof(temp_string), "%a %b %d %Y", localtime(&t)); 
  if(result!=0) {
    w_temp = XtNameToWidget(W_MainWindow, "*DateLabel");
    if(w_temp==NULL)  { 
      NonFatalError("fof.c", "SetDateValue","Unable to Retrieve DateLabel widget");
      return;
    }
    string = XmStringCreateSimple(temp_string);
    nbargs=0;
    XtSetArg(args[nbargs], XmNlabelString, string);nbargs++;
    XtSetValues(w_temp, args, nbargs);
    XmStringFree(string);
  } else NonFatalError("fof.c", "SetDateValue", "Unable to Get Date Value");
}

void InitRefreshProc(void)
{
  struct stat statbuf;
  int result;

  result = stat(".", &statbuf);
  RefreshDirLastTime=statbuf.st_ctime;
}

void TimeOutRefreshProc(XtPointer data, XtIntervalId *id)
{
  struct stat statbuf;
  int result;
  static int start=1;
  int refresh_rate;

  if(start==1) SelectDefaultViewMode();

  if(!BlockRefreshRoutine) {
    result = stat(".", &statbuf);
    if(statbuf.st_ctime!=RefreshDirLastTime || AskRefreshFileList) {
      if(start!=2) RefreshFileList();
      if(start<3 && start!=0) start++; else start=0;
      RefreshDirLastTime=statbuf.st_ctime;
      AskRefreshFileList=0;
    }
  }

  GetVarInt("RefreshDirTimeInterval", &refresh_rate);

  XtAppAddTimeOut(AppContext, refresh_rate, TimeOutRefreshProc, 0);
  
}
  
void TimeOutStartProc(XtPointer data, XtIntervalId *id)
{
  char *cwd_string;

  cwd_string = getcwd(NULL, 0);
  if(cwd_string==NULL) FatalError("fof.c","TimeOutStartProc","Cannot get current work directory (getcwd failed)");
  CreateDirTree(cwd_string);
  ActualSortType=SORT_BY_TYPE;
  SetHostValue();
  SetDateValue();
  InstallDirTreeCB();
  InitModuleFileView();
  InstallDefaultMenuCB();
  InitModuleWindows();
  XtAppAddTimeOut(AppContext, 0, TimeOutRefreshProc, 0);
}

int main(int argc, char **argv)
{

  int i;
  char *rcfile;
  char *homestr;

  InitInterface(argc, argv);

  rcfile=NULL;
  for(i=0;i<argc;i++)
    if(strcmp(argv[i],"-rc")==0)
      rcfile=argv[i+1];

  if(rcfile!=NULL)
    LoadConfigFile(rcfile);
  else {
    homestr=getenv("HOME");
    rcfile=XtMalloc(strlen(homestr)+1+7);
    strcpy(rcfile,homestr);
    strcat(rcfile,"/.fofrc");
    LoadConfigFile(rcfile);
    XtFree(rcfile);
  }
  
  if(argv[1]!=NULL && argv[1][0]!='-')
	chdir(argv[1]);
  
  CreateDefaultPixmaps();

  InitDirHistory();

  XtManageChild(W_MainWindow);

  CreateUserMenu();
  CreateContextMenus();

  InitModuleDirTree();
  InitModuleDialogs();
  InitModuleCommand();

  XtRealizeWidget(W_App);

#ifdef OFFIX_DND
  DND_Initialize(W_App);
#endif
 
 
  UserID=getuid();
  GroupID=getgid();
  TimeOutStart = 1000;
  AskRefreshFileList=0;
  BlockRefreshRoutine=0;

  XtAppAddTimeOut(AppContext, TimeOutStart, TimeOutStartProc, 0);
  XtAppMainLoop(AppContext);
  return(0);
}












